package cz.muni.fi.pa165.entity;

import cz.muni.fi.pa165.enums.CountryEnum;
import cz.muni.fi.pa165.enums.TournamentFormat;
import lombok.Getter;
import lombok.Setter;

import javax.persistence.*;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

/**
 * @author mvalko
 */

@Getter
@Setter
@Entity
public class TournamentEntity extends BaseEntity {

    @Column(nullable = false, unique = true)
    private String city;

    @Column(nullable = false)
    private String name;
    private CountryEnum country;
    private double prizePool;
    private TournamentFormat tournamentFormat;
    private int teamCapacity;

    @ManyToOne
    @JoinColumn(name = "winner_id")
    private TeamEntity winner;

    @ManyToMany
    @JoinTable(
            name="teams_enrolled",
            joinColumns = @JoinColumn(name = "tournament_id_enrolled"),
            inverseJoinColumns = @JoinColumn(name = "team_id")
    )
    private Set<TeamEntity> teamsEnrolled = new HashSet<>();


    @ManyToMany
    @JoinTable(
            name="teams_signed",
            joinColumns = @JoinColumn(name = "tournament_id_signed"),
            inverseJoinColumns = @JoinColumn(name = "team_id")
    )
    private Set<TeamEntity> teamsSigned = new HashSet<>();

    @ManyToOne
    private TournamentOrganizerEntity tournamentOrganizer;

    @OneToMany(mappedBy = "tournament")
    private Set<MatchEntity> matches = new HashSet<>();

    public TournamentEntity() {
    }

    public TournamentEntity(String name, String city, CountryEnum country,
                            int pricePool, TournamentFormat format, int capacity) {

        this.name = name;
        this.city = city;
        this.country = country;
        this.prizePool = pricePool;
        this.tournamentFormat = format;
        this.teamCapacity = capacity;
    }


    /**
     * Adds team to the tournament if team is already signed for the event and capacity of an event allows it.
     * @param team Team to be added to the tournament.
     * @return Result of team' addition to the tournament.
     */
    public boolean addTeamToTournament(TeamEntity team){
        if (teamsEnrolled.size() < teamCapacity) {
            teamsEnrolled.add(team);
            return true;
        }

        return false;
    }

    /**
     * Allows coach to sign team for the tournament.
     * @param team Team to be signed for the tournament.
     */
    public void signTeamForTournament(TeamEntity team) {
        teamsSigned.add(team);
    }

}
