package cz.muni.fi.pa165.entity;

import cz.muni.fi.pa165.enums.CountryEnum;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestInstance;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import java.time.LocalDate;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNull;

/**
 * @author Marek Kadlecik
 */
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
public class PlayerEntityTest {

    private final EntityManagerFactory emf = Persistence.createEntityManagerFactory("default");

    private TeamEntity team;
    private PlayerEntity playerA;
    private PlayerEntity playerB;
    private PlayerEntity gambler;
    private final LocalDate BIRTH_DATE = LocalDate.of(2022,2,2);


    @BeforeAll
    void init() {
        team = new TeamEntity();
        team.setName("Autisti");
        playerA = new PlayerEntity("Player A");
        playerB = new PlayerEntity("Player B");
        gambler = new PlayerEntity("SlotMachiner");

        team.addPlayerToTheTeam(playerA);
        team.addPlayerToTheTeam(playerB);
        playerA.setTeam(team);
        playerB.setTeam(team);
        gambler.setKillStat(1000);
        gambler.setAssistStat(500);
        gambler.setDeathStat(0);
        gambler.setCountry(CountryEnum.INDIA);
        gambler.setDateOfBirth(BIRTH_DATE);

        EntityManager em = emf.createEntityManager();
        try {
            em.getTransaction().begin();
            em.persist(team);
            em.persist(playerA);
            em.persist(playerB);
            em.persist(gambler);
            em.getTransaction().commit();
        } finally {
            if (em != null) em.close();
        }
    }

    @Test
    void attributesTest() {
        PlayerEntity player;
        EntityManager em = null;
        try {
            em = emf.createEntityManager();
            em.getTransaction().begin();
            player = em.find(PlayerEntity.class, gambler.getId());
            assertNull(player.getTeam());
            assertEquals(player.getKillStat(), 1000);
            assertEquals(player.getAssistStat(), 500);
            assertEquals(player.getDeathStat(), 0);
            assertEquals(player.getCountry(), CountryEnum.INDIA);
            assertEquals(player.getDateOfBirth(), BIRTH_DATE);
            em.getTransaction().commit();
        } finally {
            if (em != null) em.close();
        }
    }

    @Test
    void relationshipPlayerA() {
        PlayerEntity player;
        EntityManager em = null;
        try {
            em = emf.createEntityManager();
            em.getTransaction().begin();
            player = em.find(PlayerEntity.class, playerA.getId());
            assertEquals(player.getTeam(), team);
            em.getTransaction().commit();
        } finally {
            if (em != null) em.close();
        }
    }

    @Test
    void relationshipPlayerB() {
        PlayerEntity player;
        EntityManager em = null;
        try {
            em = emf.createEntityManager();
            em.getTransaction().begin();
            player = em.find(PlayerEntity.class, playerB.getId());
            assertEquals(player.getTeam(), team);
            em.getTransaction().commit();
        } finally {
            if (em != null) em.close();
        }
    }

    @Test
    void relationshipTeam() {
        TeamEntity team;
        EntityManager em = null;
        try {
            em = emf.createEntityManager();
            em.getTransaction().begin();
            team = em.find(TeamEntity.class, this.team.getId());
            assertEquals(team.getPlayerList().size(), 2);
            em.getTransaction().commit();
        } finally {
            if (em != null) em.close();
        }
    }

    @AfterAll
    public void afterAll() {
        EntityManager em = null;
        try {
            em = emf.createEntityManager();
            em.getTransaction().begin();
            em.createQuery("delete from PlayerEntity").executeUpdate();
            em.getTransaction().commit();
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }

}
