package cz.muni.fi.pa165.entity;

import cz.muni.fi.pa165.enums.CountryEnum;
import cz.muni.fi.pa165.enums.TournamentFormat;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestInstance;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import java.util.HashSet;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;

/**
 * @author Marek Kadlečík
 */
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
public class TournamentEntityTest {

    private final EntityManagerFactory emf = Persistence.createEntityManagerFactory("default");
    private TournamentEntity tournament;
    private TournamentEntity relational;
    private TeamEntity teamA;
    private TeamEntity teamB;

    @BeforeAll
    void init() {
        tournament = new TournamentEntity();
        tournament.setName("MSI 2");
        tournament.setTournamentFormat(TournamentFormat.DEFAULT);
        tournament.setCountry(CountryEnum.SLOVAKIA);
        tournament.setCity("Pardubice");
        tournament.setPrizePool(5);
        tournament.setTeamCapacity(30);

        relational = new TournamentEntity();
        relational.setName("MSI");
        relational.setCity("Beijing");
        relational.setTeamCapacity(20);
        relational.setCountry(CountryEnum.CHINA);

        teamA = new TeamEntity();
        teamA.setName("Alfa");
        teamB = new TeamEntity();
        teamB.setName("Beta");

        EntityManager em = null;
        try {
            em = emf.createEntityManager();
            em.getTransaction().begin();
            em.persist(tournament);
            em.persist(teamA);
            em.persist(teamB);
            relational.setTeamsEnrolled(new HashSet<>(List.of(teamA, teamB)));
            teamA.setCompletedTournaments(new HashSet<>(List.of(relational)));
            teamB.setCompletedTournaments(new HashSet<>(List.of(relational)));
            em.persist(relational);
            em.getTransaction().commit();
        } finally {
            if (em != null) em.close();
        }
    }

    @Test
    void attributesTests() {
        TournamentEntity tournament1;
        EntityManager em1 = null;
        try {
            em1 = emf.createEntityManager();
            em1.getTransaction().begin();
            tournament1 = em1.find(TournamentEntity.class, tournament.getId());
            em1.getTransaction().commit();
        } finally {
            if (em1 != null) em1.close();
        }
        assertEquals(tournament1.getTournamentFormat(), TournamentFormat.DEFAULT);
        assertEquals(tournament1.getCountry(), CountryEnum.SLOVAKIA);
        assertEquals(tournament1.getCity(), "Pardubice");
        assertEquals(tournament1.getPrizePool(), 5);
        assertEquals(tournament1.getTeamCapacity(), 30);
    }

    @Test
    void relationshipTests() {
        TournamentEntity tournament;
        EntityManager em = null;
        try {
            em = emf.createEntityManager();
            em.getTransaction().begin();
            tournament = em.find(TournamentEntity.class, relational.getId());
            assertEquals(tournament.getTeamsEnrolled(), new HashSet<>(List.of(teamA, teamB)));
            em.getTransaction().commit();
        } finally {
            if (em != null) em.close();
        }
    }
    @AfterAll
    public void afterAll() {
        EntityManager em = null;
        try {
            em = emf.createEntityManager();
            em.getTransaction().begin();
            em.createQuery("delete from TournamentEntity").executeUpdate();
            em.getTransaction().commit();
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }
}
