package cz.muni.fi.pa165.entity;

import cz.muni.fi.pa165.enums.CountryEnum;
import cz.muni.fi.pa165.enums.TournamentFormat;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestInstance;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import java.util.HashSet;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;

/**
 *  @author xpiacek
 */
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
public class TournamentOrganizerEntityTest {

    private final EntityManagerFactory emf = Persistence.createEntityManagerFactory("default");

    private TournamentEntity tournamentA;
    private TournamentEntity tournamentB;
    private TournamentEntity tournamentC;
    private TournamentEntity tournamentD;
    private TournamentEntity tournamentE;
    private TournamentOrganizerEntity organizerA;
    private TournamentOrganizerEntity organizerB;

    @BeforeAll
    void init() {
        tournamentA = new TournamentEntity();
        tournamentA.setName("MSI");
        tournamentA.setTournamentFormat(TournamentFormat.DEFAULT);
        tournamentA.setCountry(CountryEnum.SLOVAKIA);
        tournamentA.setCity("Zilina");
        tournamentA.setPrizePool(50);
        tournamentA.setTeamCapacity(30);

        tournamentB = new TournamentEntity();
        tournamentB.setName("MSI 2");
        tournamentB.setTournamentFormat(TournamentFormat.DEFAULT);
        tournamentB.setCountry(CountryEnum.CZECH_REPUBLIC);
        tournamentB.setCity("Kolin");
        tournamentB.setPrizePool(15);
        tournamentB.setTeamCapacity(2);

        tournamentC = new TournamentEntity();
        tournamentC.setName("MSI 3");
        tournamentC.setTournamentFormat(TournamentFormat.DEFAULT);
        tournamentC.setCountry(CountryEnum.ZIMBABWE);
        tournamentC.setCity("Zimbibumbo");
        tournamentC.setPrizePool(5000);
        tournamentC.setTeamCapacity(20);

        organizerA = new TournamentOrganizerEntity("AutistaOrganizer");
        organizerA.setCountry(CountryEnum.SLOVAKIA);

        tournamentA.setTournamentOrganizer(organizerA);
        tournamentB.setTournamentOrganizer(organizerA);
        tournamentC.setTournamentOrganizer(organizerA);
        organizerA.setOrganizedTournaments(new HashSet<>(List.of(tournamentA, tournamentC)));
        organizerA.setFutureTournaments(new HashSet<>(List.of(tournamentB)));

        tournamentD = new TournamentEntity("MSI","Prievidza", CountryEnum.SLOVAKIA, 1, TournamentFormat.DEFAULT, 3);
        tournamentE = new TournamentEntity("MSI 2","Osaka", CountryEnum.SLOVAKIA, 1000, TournamentFormat.DEFAULT, 10);
        organizerB = new TournamentOrganizerEntity("AutistaSpomaleny");
        organizerB.setCountry(CountryEnum.UGANDA);
        organizerB.addOrganizedTournament(tournamentD);
        organizerB.addFutureTournament(tournamentE);
        tournamentD.setTournamentOrganizer(organizerB);
        tournamentE.setTournamentOrganizer(organizerB);

        EntityManager em = emf.createEntityManager();
        try {
            em.getTransaction().begin();
            em.persist(tournamentA);
            em.persist(tournamentB);
            em.persist(tournamentC);
            em.persist(tournamentD);
            em.persist(tournamentE);
            em.persist(organizerA);
            em.persist(organizerB);

            em.getTransaction().commit();
        } finally {
            if (em != null) em.close();
        }
    }
    /**
     * Checks attributes of tournament[A-C] that were persisted into DB.
     */
    @Test
    void attributesTests() {
        TournamentEntity tournament;
        TournamentOrganizerEntity organizer;
        EntityManager em1 = null;
        try {
            em1 = emf.createEntityManager();
            em1.getTransaction().begin();
            tournament = em1.find(TournamentEntity.class, tournamentA.getId());
            assertEquals(tournament.getTournamentFormat(), TournamentFormat.DEFAULT);
            assertEquals(tournament.getCountry(), CountryEnum.SLOVAKIA);
            assertEquals(tournament.getCity(), "Zilina");
            assertEquals(tournament.getName(), "MSI");
            assertEquals(tournament.getPrizePool(), 50);
            assertEquals(tournament.getTeamCapacity(), 30);

            tournament = em1.find(TournamentEntity.class, tournamentB.getId());
            assertEquals(tournament.getTournamentFormat(), TournamentFormat.DEFAULT);
            assertEquals(tournament.getCountry(), CountryEnum.CZECH_REPUBLIC);
            assertEquals(tournament.getCity(), "Kolin");
            assertEquals(tournament.getName(), "MSI 2");
            assertEquals(tournament.getPrizePool(), 15);
            assertEquals(tournament.getTeamCapacity(), 2);

            tournament = em1.find(TournamentEntity.class, tournamentC.getId());
            assertEquals(tournament.getTournamentFormat(), TournamentFormat.DEFAULT);
            assertEquals(tournament.getCountry(), CountryEnum.ZIMBABWE);
            assertEquals(tournament.getCity(), "Zimbibumbo");
            assertEquals(tournament.getName(), "MSI 3");
            assertEquals(tournament.getPrizePool(), 5000);
            assertEquals(tournament.getTeamCapacity(), 20);

            organizer = em1.find(TournamentOrganizerEntity.class, organizerA.getId());
            assertEquals(organizer.getName(), "AutistaOrganizer");
            assertEquals(organizer.getCountry(), CountryEnum.SLOVAKIA);

            em1.getTransaction().commit();
        } finally {
            if (em1 != null) em1.close();
        }
    }

    /**
     * Checks tournament organizer that was persisted into DB.
     */
    @Test
    void relationshipTournamentEntity() {
        TournamentEntity tournament;
        EntityManager em = null;
        try {
            em = emf.createEntityManager();
            em.getTransaction().begin();
            tournament = em.find(TournamentEntity.class, tournamentA.getId());
            assertEquals(tournament.getTournamentOrganizer(),organizerA);

            tournament = em.find(TournamentEntity.class, tournamentB.getId());
            assertEquals(tournament.getTournamentOrganizer(),organizerA);

            em.getTransaction().commit();
        } finally {
            if (em != null) em.close();
        }
    }

    /**
     * Checks organizer's organized and future tournaments that were persisted into DB.
     */
    @Test
    void relationshipTournamentOrganizerEntity() {
        TournamentOrganizerEntity organizer;
        EntityManager em = null;
        try {
            em = emf.createEntityManager();
            em.getTransaction().begin();
            organizer = em.find(TournamentOrganizerEntity.class, organizerA.getId());
            assertEquals(organizer.getOrganizedTournaments(), new HashSet<>(List.of(tournamentA, tournamentB, tournamentC)));
            assertEquals(organizer.getFutureTournaments(), new HashSet<>(List.of(tournamentB, tournamentA, tournamentC)));

            organizer = em.find(TournamentOrganizerEntity.class, organizerB.getId());
            assertEquals(organizer.getOrganizedTournaments(), new HashSet<>(List.of(tournamentD, tournamentE)));
            assertEquals(organizer.getFutureTournaments(), new HashSet<>(List.of(tournamentE, tournamentD)));
            em.getTransaction().commit();
        } finally {
            if (em != null) em.close();
        }
    }

    @AfterAll
    public void afterAll() {
        EntityManager em = null;
        try {
            em = emf.createEntityManager();
            em.getTransaction().begin();
            em.createQuery("delete from TournamentEntity").executeUpdate();
            em.createQuery("delete from TournamentOrganizerEntity").executeUpdate();
            em.getTransaction().commit();
        } finally {
            if (em != null) {
                em.close();
            }
        }
    }
}
