package cz.muni.fi.pa165.rest.controllers;

import cz.muni.fi.pa165.dto.PlayerDTO;
import cz.muni.fi.pa165.facade.PlayerFacade;
import cz.muni.fi.pa165.rest.RootUris;
import cz.muni.fi.pa165.rest.exception.ResourceAlreadyExistsException;
import cz.muni.fi.pa165.rest.exception.ResourceNotFoundException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

@RestController
@RequestMapping(RootUris.restPlayer)
public class PlayerController {

    @Autowired
    private PlayerFacade playerFacade;

    @RequestMapping(value = "/{uuid}", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public final PlayerDTO findById(@PathVariable("uuid") String uuid) {
        var playerDTO = playerFacade.getPlayerById(uuid);
        if (playerDTO != null) {
            return playerDTO;
        } else {
            throw new ResourceNotFoundException();
        }
    }

    @RequestMapping(method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE)
    public final List<PlayerDTO> findAll() {
        return playerFacade.findAllPlayers();
    }

    @RequestMapping(value = "/create", method = RequestMethod.POST, consumes = MediaType.APPLICATION_JSON_VALUE,
            produces = MediaType.APPLICATION_JSON_VALUE)
    public final PlayerDTO create(@RequestBody PlayerDTO player) {
        try {
            String  uuid = playerFacade.createPlayer(player);
            return playerFacade.getPlayerById(uuid);
        } catch (Exception e) {
            throw new ResourceAlreadyExistsException();
        }
    }

    @RequestMapping(value = "/{uuid}", method = RequestMethod.PUT)
    public void updateSteward(@PathVariable("uuid") String uuid, @RequestBody PlayerDTO playerDTO) throws ResourceNotFoundException {
        try {
            playerDTO.setId(uuid);
            playerFacade.editPlayer(playerDTO);
        } catch (Exception exception) {
            throw new ResourceNotFoundException();
        }
    }

    @RequestMapping(value = "/{uuid}", method = RequestMethod.DELETE)
    public final void delete(@PathVariable("uuid") String uuid) throws ResourceNotFoundException {
        try {
            playerFacade.delete(uuid);
        } catch (Exception ex) {
            throw new ResourceNotFoundException();
        }
    }
}
