package cz.muni.fi.pa165.facade;

import cz.muni.fi.pa165.dto.PersonAuthenticateDTO;
import cz.muni.fi.pa165.dto.PersonDTO;
import cz.muni.fi.pa165.entity.PersonEntity;
import cz.muni.fi.pa165.services.BeanMappingService;
import cz.muni.fi.pa165.services.person.PersonService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
@Transactional
public class PersonFacadeImpl implements PersonFacade {

    @Autowired
    private PersonService personService;

    @Autowired
    private BeanMappingService beanMappingService;

    @Override
    public List<PersonDTO> findAllPeople() {
        return beanMappingService.mapTo(personService.findAll(), PersonDTO.class);
    }

    @Override
    public PersonDTO getPersonById(String uuid) {
        PersonEntity person = personService.findById(uuid);
        return (person == null) ? null : beanMappingService.mapTo(person, PersonDTO.class);
    }

    @Override
    public PersonDTO getPersonByUsername(String username) {
        PersonEntity person = personService.findByUsername(username);
        return (person == null) ? null : beanMappingService.mapTo(person, PersonDTO.class);
    }

    @Override
    public void delete(String uuid) {
        var player = new PersonEntity();
        player.setId(uuid);
        personService.remove(player);
    }

    @Override
    public boolean authenticate(PersonAuthenticateDTO person) {
        return false;
    }

    @Override
    public boolean isAdmin(PersonDTO person) {
        return personService.isAdmin(beanMappingService.mapTo(person, PersonEntity.class));
    }
}
