package cz.muni.fi.pa165.facade;

import cz.muni.fi.pa165.dto.PlayerDTO;
import cz.muni.fi.pa165.entity.PlayerEntity;
import cz.muni.fi.pa165.services.BeanMappingService;
import cz.muni.fi.pa165.services.player.PlayerService;
import cz.muni.fi.pa165.services.team.TeamService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
@Transactional
public class PlayerFacadeImpl implements PlayerFacade {

    @Autowired
    private PlayerService playerService;

    @Autowired
    private TeamService teamService;
    @Autowired
    private BeanMappingService beanMappingService;

    @Override
    public List<PlayerDTO> findAllPlayers() {
        return beanMappingService.mapTo(playerService.findAll(), PlayerDTO.class);
    }

    @Override
    public PlayerDTO getPlayerById(String uuid) {
        PlayerEntity player = playerService.findById(uuid);
        return (player == null) ? null : beanMappingService.mapTo(player, PlayerDTO.class);
    }

    @Override
    public String createPlayer(PlayerDTO playerDTO) {

        PlayerEntity player = new PlayerEntity();

        player.setName(playerDTO.getName());
        player.setCountry(playerDTO.getCountry());
        player.setTeam(teamService.findById(playerDTO.getTeamId()));
        player.setKillStat(playerDTO.getKillStat());
        player.setDeathStat(playerDTO.getDeathStat());
        player.setAssistStat(playerDTO.getAssistStat());

        playerService.create(player);
        return player.getId();
    }

    @Override
    public String editPlayer(PlayerDTO playerDTO) {

        PlayerEntity player = playerService.findById(playerDTO.getId());

        player.setName(playerDTO.getName());
        player.setCountry(playerDTO.getCountry());
        player.setTeam(teamService.findById(playerDTO.getTeamId()));
        player.setKillStat(playerDTO.getKillStat());
        player.setDeathStat(playerDTO.getDeathStat());
        player.setAssistStat(playerDTO.getAssistStat());

        playerService.update(player);
        return player.getId();
    }

    @Override
    public void delete(String uuid) {
        var player = new PlayerEntity();
        player.setId(uuid);
        playerService.remove(player);
    }
}
