package cz.muni.fi.pa165.services.person;

import com.google.common.hash.Hashing;
import cz.muni.fi.pa165.dao.person.PersonDao;
import cz.muni.fi.pa165.entity.PersonEntity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.argon2.Argon2PasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.UUID;

/**
 * @author Dominik Kozubík
 */
@Service
@Transactional
public class PersonServiceImpl implements PersonService {

    @Autowired
    private PersonDao personDao;

    @Override
    public void create(PersonEntity person) {
        if (person == null) {
            throw new IllegalArgumentException("Given entity is null.");
        }
        personDao.create(person);
    }

    @Override
    public List<PersonEntity> findAll() {
        return personDao.findAll();
    }

    @Override
    public PersonEntity findById(String id) {
        if (id == null) {
            throw new IllegalArgumentException("Given id is null.");
        }
        return personDao.findById(id);
    }

    @Override
    public PersonEntity findByUsername(String username) {
        if (username == null) {
            throw new IllegalArgumentException("Given username is null.");
        }
        return personDao.findByUsername(username);
    }

    @Override
    public boolean authenticate(PersonEntity person, String password) {
        String sha256hex = Hashing.sha256().hashString(password, StandardCharsets.UTF_8).toString();
        return person.getPasswordHash().equals(sha256hex);
    }

    @Override
    public boolean isAdmin(PersonEntity person) {
        return findById(person.getId()).isAdmin();
    }

    @Override
    public String update(PersonEntity person) {
        if (person == null) {
            throw new IllegalArgumentException("Given entity is null.");
        }
        return personDao.update(person);
    }

    @Override
    public void remove(PersonEntity person) {
        if (person == null) {
            throw new IllegalArgumentException("Given entity is null.");
        }
        personDao.remove(person);
    }
}
