package cz.muni.fi.pa165;

import cz.muni.fi.pa165.config.ServiceConfiguration;
import cz.muni.fi.pa165.dao.teamCoach.TeamCoachDao;
import cz.muni.fi.pa165.entity.TeamCoachEntity;
import cz.muni.fi.pa165.services.teamCoach.TeamCoachService;
import cz.muni.fi.pa165.services.teamCoach.TeamCoachServiceImpl;
import org.hibernate.service.spi.ServiceException;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.List;
import java.util.UUID;

import static java.util.Arrays.asList;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

/**
 * @author Adrian Piacek, Marek Kadlencik
 */
@ContextConfiguration(classes = ServiceConfiguration.class)
public class TeamCoachServiceTest {
    @Mock
    private TeamCoachDao teamCoachDao;

    @Autowired
    @InjectMocks
    private TeamCoachService teamCoachService = new TeamCoachServiceImpl();

    private TeamCoachEntity teamCoachA;

    @BeforeClass
    public void setup() throws ServiceException {
        MockitoAnnotations.openMocks(this);
    }

    @BeforeMethod
    public void createTestTeamCoach() {
        teamCoachA = new TeamCoachEntity("Autista TeamCoach");
        teamCoachA.setId(UUID.randomUUID().toString());
    }


    @Test
    public void createTeamCoach() {
        teamCoachService.create(teamCoachA);
        Mockito.verify(teamCoachDao, Mockito.times(1)).create(teamCoachA);
    }

    @Test
    public void createNullTeamCoach() {
        Assert.assertThrows(IllegalArgumentException.class, () -> teamCoachService.create(null));
    }

    @Test
    public void findTeamCoachById() {
        when(teamCoachDao.findById(teamCoachA.getId())).thenReturn(teamCoachA);

        TeamCoachEntity result = teamCoachService.findById(teamCoachA.getId());

        Assert.assertEquals(result.getId(), teamCoachA.getId());
    }

    @Test
    public void findTeamCoachByNullId() {
        Assert.assertThrows(IllegalArgumentException.class, () -> teamCoachService.findById(null));
    }


    @Test
    public void findAllTTeamCoaches() {
        TeamCoachEntity teamCoachB = new TeamCoachEntity("Autista TeamCoach Pomocnik");
        when(teamCoachDao.findAll()).thenReturn(asList(teamCoachB, teamCoachA));

        List<TeamCoachEntity> result = teamCoachService.findAll();

        Assert.assertEquals(result.size(), 2);
    }

    @Test
    public void deleteTeamCoach() {
        teamCoachService.remove(teamCoachA);

        verify(teamCoachDao, times(1)).remove(teamCoachA);
    }

    @Test
    public void deleteNullTeamCoach() {
        Assert.assertThrows(IllegalArgumentException.class, () -> teamCoachService.remove(null));
    }

    @Test
    public void updateTeamCoach() {
        TeamCoachEntity teamCoachB = new TeamCoachEntity("Autista TeamCoach Pomocnik");
        teamCoachService.update(teamCoachB);

        verify(teamCoachDao, times(1)).update(teamCoachB);
    }

    @Test
    public void updateNullTeamCoach() {
        Assert.assertThrows(IllegalArgumentException.class, () -> teamCoachService.update(null));
    }

}
