package cz.muni.fi.pa165.mvc.controllers;

import cz.muni.fi.pa165.dto.PlayerDTO;
import cz.muni.fi.pa165.dto.TeamDTO;
import cz.muni.fi.pa165.enums.CountryEnum;
import cz.muni.fi.pa165.facade.PlayerFacade;
import cz.muni.fi.pa165.facade.TeamFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;
import org.springframework.web.util.UriComponentsBuilder;

import javax.validation.Valid;
import java.util.List;

/**
 * @author Matus Valko
 */
@Controller
@RequestMapping("/team")
public class TeamController {

    @Autowired
    private TeamFacade teamFacade;

    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public String list(Model model) {
        List<TeamDTO> teams = teamFacade.findAllTeams();
        model.addAttribute("teams", teams);
        return "team/list";
    }

    @RequestMapping(value = "/new", method = RequestMethod.GET)
    public String newTeam(Model model) {
        model.addAttribute("teamCreate", new TeamDTO());
        return "team/new";
    }

    @RequestMapping(value = "/view/{id}", method = RequestMethod.GET)
    public String view(@PathVariable String id, Model model) {
        model.addAttribute("team", teamFacade.getTeamById(id));
        model.addAttribute("teamPlayers", teamFacade.getTeamById(id).getPlayerList());
        return "team/view";
    }

    @ModelAttribute("countries")
    public CountryEnum[] countries() {
        return CountryEnum.values();
    }

    @RequestMapping(value = "/create", method = RequestMethod.POST)
    public String create(@Valid @ModelAttribute("playerCreate") TeamDTO formBean, BindingResult bindingResult,
                         Model model, RedirectAttributes redirectAttributes, UriComponentsBuilder uriBuilder) {
        //in case of validation error forward back to the the form
        if (bindingResult.hasErrors()) {
            return "team/new";
        }
        //create product
        String id = teamFacade.createTeam(formBean);
        //report success
        redirectAttributes.addFlashAttribute("alert_success", "Team " + formBean.getName() + " was created");
        return "redirect:" + uriBuilder.path("/team/view/{id}").buildAndExpand(id).encode().toUriString();
    }

    @RequestMapping(value = "/delete/{uuid}", method = RequestMethod.POST)
    public String delete(@PathVariable String uuid, UriComponentsBuilder uriBuilder, RedirectAttributes redirectAttributes) {
        TeamDTO team = teamFacade.getTeamById(uuid);
        try {
            teamFacade.delete(uuid);
            redirectAttributes.addFlashAttribute("alert_success", "Team \"" + team.getName() + "\" was deleted.");
        } catch (Exception ex) {
            redirectAttributes.addFlashAttribute("alert_danger", "Team \"" + team.getName() + "\" cannot be deleted.");
        }
        return "redirect:" + uriBuilder.path("/team/list").toUriString();
    }
}
